@@ -4,5 +4,5 @@ rm -rf dist
 #python setup.py build_ext --inplace
 #python setup.py install
 python setup.py sdist
-pip install dist/*.tar.gz
+pip install dist/*.tar.gz --force-reinstall
 rm -rf ./*.egg-info
\ No newline at end of file
