@@ -183,6 +183,8 @@ def create_c3nn_classifier(ninput=100, nfilters=32, kernel_size=4, ndense=128, p
         dropout rate
     noutput:
         output shape
+    pooling:
+        if True, add pooling layers
 
     Returns
     -------
@@ -211,7 +213,10 @@ def create_c3nn_classifier(ninput=100, nfilters=32, kernel_size=4, ndense=128, p
     model.add(Dense(ndense, activation="relu", ))  # input_shape=(4000,)
     model.add(BatchNormalization())
 
-    model.add(Dense(noutput, activation="sigmoid"))
+    if noutput == 1:
+        model.add(Dense(noutput, activation="sigmoid"))
+    else:
+        model.add(Dense(noutput, activation="softmax"))
     return model
 
 
